library async_utils;

import 'package:flutter/widgets.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

part 'async_hooks.freezed.dart';

@freezed
class AsyncValue<T> with _$AsyncValue {
  const factory AsyncValue.loading() = AsyncLoading;
  const factory AsyncValue.error(Object? error) = AsyncError;
  const factory AsyncValue.data(
    T value, {
    @Default(true) bool isDone,
  }) = AsyncData<T>;
}

AsyncValue<T> useFutureValue<T>(Future<T> future) {
  final snapshot = useFuture(future);

  switch (snapshot.connectionState) {
    case ConnectionState.none:
    case ConnectionState.waiting:
      return const AsyncValue.loading();
    case ConnectionState.active:
    case ConnectionState.done:
      return AsyncValue.data(snapshot.data as T, isDone: true);
  }
}

AsyncValue<T> useStreamValue<T>(Stream<T> stream) {
  final snapshot = useStream(stream);

  switch (snapshot.connectionState) {
    case ConnectionState.none:
    case ConnectionState.waiting:
      return const AsyncValue.loading();
    case ConnectionState.active:
      return AsyncValue.data(snapshot.data as T, isDone: false);
    case ConnectionState.done:
      return AsyncValue.data(snapshot.data as T, isDone: true);
  }
}
